package com.hz.pm.api.workbench.manage;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.statemachine.event.*;
import com.hz.pm.api.common.util.DecimalUtil;
import com.hz.pm.api.projectdeclared.model.entity.Contract;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.enumerization.BidTypeEnum;
import com.hz.pm.api.projectdeclared.service.IContractService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.ProjectGovSystemReplaceInfos;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.manage.ProjectLibManage;
import com.hz.pm.api.projectlib.model.dto.GovSystemReplaceInfoDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.GovSystemReplaceTypeEnum;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.model.vo.SystemReplaceInfoVO;
import com.hz.pm.api.projectlib.model.vo.UnitProjectGovSystemReplaceVO;
import com.hz.pm.api.projectlib.model.vo.UnitProjectGovSystemReplaceVO.ProjectGovSystemReplaceVO;
import com.hz.pm.api.projectlib.service.IProjectGovSystemReplaceInfosService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.hz.pm.api.sys.manage.EarlyWarningManage;
import com.hz.pm.api.sys.model.entity.WflowEarlyWarningRecords;
import com.hz.pm.api.sys.service.IEarlyWarningRecordsService;
import com.hz.pm.api.todocenter.manage.TodoCenterManage;
import com.hz.pm.api.todocenter.model.req.ToBeProcessedReq;
import com.hz.pm.api.todocenter.model.vo.ResToBeProcessedVO;
import com.hz.pm.api.todocenter.model.vo.TodoCenterStatisticsVO;
import com.hz.pm.api.todocenter.model.vo.TodoVO;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.hz.pm.api.workbench.converter.WorkbenchConverter;
import com.hz.pm.api.workbench.model.WorkbenchProcessNode;
import com.hz.pm.api.workbench.model.WorkbenchProjectLibReq;
import com.hz.pm.api.workbench.model.WorkbenchReq;
import com.hz.pm.api.workbench.model.vo.ProjectProcessStatVO;
import com.hz.pm.api.workbench.model.vo.ProjectTotalViewVO;
import com.hz.pm.api.workbench.model.vo.SystemReplaceStatVO;
import com.hz.pm.api.workbench.model.vo.WorkbenchVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.StrPool;
import com.wflow.bean.entity.WflowEarlyWarning;
import com.wflow.service.IEarlyWarningService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.math3.util.Pair;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.BiFunction;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import static com.hz.pm.api.projectlib.helper.ProjectManageUtil.projectQueryByProjectStatusChange;
import static com.hz.pm.api.workbench.model.vo.ProjectProcessStatVO.TenderAdaptFinishStatVO;

/**
 * <p>
 * WorkbenchManage
 * </p>
 *
 * @author WendyYang
 * @since 18:58 2024/3/23
 */
@Slf4j
@Component
@AllArgsConstructor
public class WorkbenchManage {

    private final TodoCenterManage todoCenterManage;
    private final IEarlyWarningRecordsService earlyWarningRecordsService;
    private final IEarlyWarningService earlyWarningService;
    private final UserInfoHelper userInfoHelper;
    private final ProjectLibManage projectLibManage;
    private final EarlyWarningManage earlyWarningManage;
    private final IProjectService projectService;
    private final IProjectStatusChangeService projectStatusChangeService;
    private final IPurchaseService purchaseService;
    private final IContractService contractService;
    private final MhUnitCache mhUnitCache;
    private final IProjectGovSystemReplaceInfosService systemReplaceInfosService;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;

    public WorkbenchVO getWorkbenchData(Integer year) {
        WorkbenchVO res = new WorkbenchVO();
        Long userId = LoginUserUtil.getUserId();
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(userId);
        // 待办中心数据
        TodoCenterStatisticsVO todoStat = todoCenterManage.todoCenterStatistics(userId);
        ToBeProcessedReq todoReq = new ToBeProcessedReq();
        todoReq.page(1, 50);
        todoReq.setLoginUserId(userId);
        todoReq.setUserId(userId);
        PageVo<ResToBeProcessedVO> todoPage = todoCenterManage.todoProjectList(todoReq);
        List<TodoVO> todoList = CollUtils.fieldList(todoPage.getRecords(), WorkbenchConverter::convert);
        todoStat.setTodoList(todoList);
        res.setTodoCerter(todoStat);
        // 项目列表
        ProjectListReq projReq = new ProjectListReq();
        projReq.page(1, 20);
        projReq.setProjectYear(year);
        PageVo<ProjectLibListItemVO> projPage = projectLibManage.listWithPermissionWorkBench(projReq, user);
        res.setProjects(new ArrayList<>(projPage.getRecords()));
        return res;
    }

    /**
     * 催办
     *
     * @param recordId \
     * @return \
     */
    public synchronized String urging(Long recordId) {
        WflowEarlyWarningRecords warningRecords = earlyWarningRecordsService.getById(recordId);
        Assert.notNull(warningRecords, "催办失败,id不存在!");
        String projectCode = warningRecords.getProjectCode();
        Project project = projectService.getNoNull(projectCode);
        String batchEmployees = warningRecords.getBatchUserIds();
        Integer ruleType = warningRecords.getRuleType();
        String areaCode = warningRecords.getAreaCode();
        //如果是临期 可以取到 超期的小时数
        String path = warningRecords.getPath();
        LocalDateTime instStart = warningRecords.getInstStart();
        Integer biz = warningRecords.getBiz();
        if (StringUtils.isBlank(batchEmployees)) {
            //如果批次员工号不存在 就取当前
            batchEmployees = warningRecords.getWarningUserId();
        }
        Assert.notEmpty(batchEmployees, "催办失败,员工为空!");
        String[] employees = batchEmployees.split(StrPool.COMMA);
        String noticeContent = warningRecords.getNoticeContent();
        //查出配置的模板
        WflowEarlyWarning config = earlyWarningService.getOne(Wrappers.lambdaQuery(WflowEarlyWarning.class)
                .eq(WflowEarlyWarning::getAreaCode, areaCode)
                .eq(WflowEarlyWarning::getRuleType, ruleType)
                .last(BizConst.LIMIT_1));
        //如果配置能取到 就取模板的
        if (Objects.nonNull(config)) {
            noticeContent = config.getNoticeContent();
        }

        //算出 当前和他的 小时差 临期
        long adventBetween = ChronoUnit.HOURS.between(instStart, LocalDateTime.now());
        String noticeMethod = warningRecords.getNoticeMethod();
        Integer noticeType = warningRecords.getNoticeType();
        for (String employee : employees) {
            earlyWarningManage.urging(noticeMethod, noticeContent,
                    (int) adventBetween, (int) adventBetween,
                    biz, instStart, Long.valueOf(employee),
                    project, ruleType, noticeType, path,
                    batchEmployees, warningRecords.getNodeId());
        }
        return "催办成功了" + employees.length + "个人";
    }

    private void initWorkbenchDefaultParam(WorkbenchReq req) {
        if (req.getUnitId() == null) {
            req.setUnitId(LoginUserUtil.loginUserDetail().getMhUnitId());
        }
        if (req.getProjectYear() == null) {
            req.setProjectYear(LocalDate.now().getYear());
        }
    }

    public ProjectTotalViewVO projectTotalView(WorkbenchReq req) {
        ProjectTotalViewVO retDetail = new ProjectTotalViewVO();
        initWorkbenchDefaultParam(req);
        LambdaQueryWrapper<Project> projectQuery = Wrappers.lambdaQuery(Project.class)
                .select(Project::getReviewAmount, Project::getApprovalAmount, Project::getId,
                        Project::getApprovalGovOwnFinanceAmount, Project::getApprovalGovSuperiorFinanceAmount)
                .eq(Project::getProjectYear, req.getProjectYear())
                .eq(req.getUnitStrip() != null, Project::getUnitStrip, req.getUnitStrip())
                .eq(Project::getNewest, Boolean.TRUE);
        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(req.getUnitId());
        projectQuery.in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf));
        List<Project> projects = projectService.list(projectQuery);
        if (projects.isEmpty()) {
            return retDetail;
        }
        retDetail.setTotalApprovalAmount(DecimalUtil.sum(projects, Project::getApprovalAmount));
        retDetail.setTotalReviewAmount(DecimalUtil.sum(projects, Project::getReviewAmount));
        retDetail.setCurrOrgApprovalAmount(DecimalUtil.sum(projects, Project::getApprovalGovOwnFinanceAmount));
        retDetail.setUpperOrgApprovalAmount(DecimalUtil.sum(projects, Project::getApprovalGovSuperiorFinanceAmount));
        List<Long> projectIds = CollUtils.fieldList(projects, Project::getId);
        // 合同金额
        List<Contract> contracts = contractService.list(Wrappers.lambdaQuery(Contract.class)
                .select(Contract::getTotalAmount, Contract::getId)
                .in(Contract::getProjectId, projectIds));
        retDetail.setTotalContractAmount(DecimalUtil.sum(contracts, Contract::getTotalAmount));
        // 系统替代情况统计
        Wrapper<ProjectGovSystemReplaceInfos> pgsriQuery = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .in(ProjectGovSystemReplaceInfos::getProjectId, projectIds);
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.list(pgsriQuery);
        Map<String, List<ProjectGovSystemReplaceInfos>> replaceInfosMap = new HashMap<>();
        for (ProjectGovSystemReplaceInfos systemReplaceInfo : systemReplaceInfos) {
            List<String> replaceTypes = StrUtil.split(systemReplaceInfo.getReplaceType(), ",");
            for (String replaceType : replaceTypes) {
                if (!NumberUtil.isNumber(replaceType)) {
                    replaceType = "其他";
                } else {
                    replaceType = GovSystemReplaceTypeEnum.get(Integer.parseInt(replaceType))
                            .flatMap(t -> Optional.of(t.getVal()))
                            .orElse("其他");
                }
                replaceInfosMap.computeIfAbsent(replaceType, k -> new ArrayList<>()).add(systemReplaceInfo);
            }
        }
        replaceInfosMap.remove("其他");

        List<SystemReplaceStatVO> systemReplaceStat = replaceInfosMap.entrySet().stream().map(w -> {
            String replaceType = w.getKey();
            List<ProjectGovSystemReplaceInfos> replaceInfosList = w.getValue();
            Map<String, List<ProjectGovSystemReplaceInfos>> replaceGroupProject = CollUtils.group(replaceInfosList,
                    t -> t.getProjectCode() + "#" + t.getBatchNo());
            int sourceCount = 0;
            int targetCount = 0;
            for (List<ProjectGovSystemReplaceInfos> currSystems : replaceGroupProject.values()) {
                sourceCount += CollUtil.count(CollUtils.fieldSet(currSystems, ProjectGovSystemReplaceInfos::getSourceSystem),
                        isValidSystem::test);
                targetCount += CollUtil.count(CollUtils.fieldSet(currSystems, ProjectGovSystemReplaceInfos::getTargetSystem),
                        isValidSystem::test);
            }
            return SystemReplaceStatVO.builder()
                    .sourceCount(sourceCount)
                    .targetCount(targetCount)
                    .replaceType(replaceType)
                    .build();
        }).collect(Collectors.toList());
        Pair<Integer, Integer> totalCount = countSystemReplaceInfos(systemReplaceInfos);
        retDetail.setTotalSourceSystemCount(totalCount.getKey());
        retDetail.setTotalTargetSystemCount(totalCount.getValue());
        retDetail.setSystemReplaceInfos(systemReplaceStat);
        return retDetail;
    }

    static Predicate<String> isValidSystem = x -> StrUtil.isNotBlank(x) && !"/".equals(x);

    private Pair<Integer, Integer> countReplaceSystemByProjectIds(Collection<Long> projectIds) {
        List<String> projectCodes = CollUtils.convert(projectIds, ProjectIdCodeCacheUtil::get);
        return countReplaceSystemByProjectCodes(projectCodes);
    }

    private Pair<Integer, Integer> countReplaceSystemByProjectCodes(List<String> projectCodes) {
        if (CollUtil.isEmpty(projectCodes)) {
            return Pair.create(0, 0);
        }
        Wrapper<ProjectGovSystemReplaceInfos> query = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .select(ProjectGovSystemReplaceInfos::getSourceSystem, ProjectGovSystemReplaceInfos::getTargetSystem,
                        ProjectGovSystemReplaceInfos::getProjectCode, ProjectGovSystemReplaceInfos::getBatchNo)
                .in(ProjectGovSystemReplaceInfos::getProjectCode, projectCodes);
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.list(query);
        return countSystemReplaceInfos(systemReplaceInfos);
    }

    private static Pair<Integer, Integer> countSystemReplaceInfos(List<ProjectGovSystemReplaceInfos> systemReplaceInfos) {
        Collection<Pair<Integer, Integer>> countByProject = systemReplaceInfos.stream()
                .collect(Collectors.groupingBy(w -> w.getProjectCode() + "#" + w.getBatchNo(),
                        Collectors.collectingAndThen(Collectors.toList(),
                                w -> {
                                    Set<String> sourceNames = CollUtils.fieldSet(w, ProjectGovSystemReplaceInfos::getSourceSystem);
                                    int sourceCount = CollUtil.count(sourceNames, isValidSystem::test);
                                    Set<String> targetNames = CollUtils.fieldSet(w, ProjectGovSystemReplaceInfos::getTargetSystem);
                                    int targetCount = CollUtil.count(targetNames, isValidSystem::test);
                                    return Pair.create(sourceCount, targetCount);
                                }))).values();
        int totalTargetCount = 0;
        int totalSourceCount = 0;
        for (Pair<Integer, Integer> pair : countByProject) {
            totalSourceCount += pair.getKey();
            totalTargetCount += pair.getValue();
        }
        return Pair.create(totalSourceCount, totalTargetCount);
    }


    public List<String> getProjectCodes(List<String> projectCodes, ProjectStateChangeEvent event) {
        Wrapper<ProjectStatusChange> query = Wrappers.lambdaQuery(ProjectStatusChange.class)
                .select(ProjectStatusChange::getProjectCode)
                .in(ProjectStatusChange::getProjectCode, projectCodes)
                .eq(ProjectStatusChange::getEvent, event);
        return CollUtils.fieldList(projectStatusChangeService.list(query), ProjectStatusChange::getProjectCode);
    }

    public List<ProjectProcessStatVO> projectProcessStageStatistics(WorkbenchReq req) {
        initWorkbenchDefaultParam(req);
        List<ProjectProcessStatVO> retData = new ArrayList<>();
        List<Long> allProjectIds = new ArrayList<>();
        List<String> allProjectCodes = new ArrayList<>();
        Map<String, Map<Long, Long>> purchaseEventMap = new HashMap<>();
        Map<Long, List<Purchase>> projectPurchaseCountMap = new HashMap<>();
        BiFunction<AbstractStateChangeEvent, WorkbenchProcessNode, List<Long>> computeProjectIds = (event, node) -> {
            Map<Long, Long> projBizMap = purchaseEventMap.getOrDefault(event.name(), Collections.emptyMap());
            if (node.equals(WorkbenchProcessNode.PROJECT_ADAPTION)) {
                return projectPurchaseCountMap.entrySet().stream().filter(w -> {
                    Long finishedBizCount = projBizMap.get(w.getKey());
                    return finishedBizCount != null && finishedBizCount == CollUtil.count(w.getValue(),
                            x -> BidTypeEnum.BUILD_APP.eq(x.getBidType()));
                }).map(Map.Entry::getKey).collect(Collectors.toList());
            } else {
                return projectPurchaseCountMap.entrySet().stream().filter(w -> {
                    Long finishedBizCount = projBizMap.get(w.getKey());
                    return finishedBizCount != null && finishedBizCount == w.getValue().size();
                }).map(Map.Entry::getKey).collect(Collectors.toList());
            }
        };
        // 查询完成立项备案的项目
        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(req.getUnitId());
        Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getApprovalAmount, Project::getProjectCode)
                .eq(Project::getProjectYear, req.getProjectYear())
                .eq(req.getUnitStrip() != null, Project::getUnitStrip, req.getUnitStrip())
                .eq(Project::getNewest, Boolean.TRUE)
                .exists(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE + " and npsc.event = {0}",
                        ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT)
                .in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf));
        List<Project> projects = projectService.list(query);
        Map<String, List<ProjectStatusChange>> projectStatusChangeMap = new HashMap<>();
        for (WorkbenchProcessNode node : WorkbenchProcessNode.ALL) {
            if (!projects.isEmpty()) {
                ProjectProcessStatVO currStat = ProjectProcessStatVO.builder()
                        .stageName(node.getName())
                        .stage(node)
                        .build();
                switch (node) {
                    case PROJECT_DECLARED: {
                        currStat.setProjectCount(projects.size());
                        for (Project project : projects) {
                            allProjectCodes.add(project.getProjectCode());
                            allProjectIds.addAll(ProjectIdCodeCacheUtil.get(project.getProjectCode()));
                        }
                        Wrapper<ProjectStatusChange> pscQuery = Wrappers.lambdaQuery(ProjectStatusChange.class)
                                .in(ProjectStatusChange::getProjectCode, allProjectCodes);
                        List<ProjectStatusChange> projectStatusChanges = projectStatusChangeService.list(pscQuery);
                        projectStatusChangeMap.putAll(CollUtils.group(projectStatusChanges, ProjectStatusChange::getProjectCode));
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap, allProjectCodes));
                        Pair<Integer, Integer> replaceSystemCount = countReplaceSystemByProjectIds(allProjectIds);
                        currStat.setSourceCount(replaceSystemCount.getKey());
                        currStat.setTargetCount(replaceSystemCount.getValue());
                    }
                    break;
                    case PROJECT_REVIEW: {
                        List<String> projectCodes = getProjectCodes(allProjectCodes, ProjectStateChangeEvent.PROJECT_REVIEW_PASS);
                        currStat.setProjectCount(projectCodes.size());
                        Pair<Integer, Integer> systemReplaceCount = countReplaceSystemByProjectCodes(projectCodes);
                        currStat.setSourceCount(systemReplaceCount.getKey());
                        currStat.setTargetCount(systemReplaceCount.getValue());
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap, projectCodes));
                        int reviewFailed = lastMatchProjectCount(projectStatusChangeMap, ProjectStateChangeEvent.PROJECT_REVIEW_FAILED);
                        int reviewing = lastMatchProjectCount(projectStatusChangeMap, ProjectStateChangeEvent.PROJECT_REVIEW_SUBMIT,
                                ProjectStateChangeEvent.PROJECT_REVIEW_RESUBMIT);
                        currStat.setReviewingCount(reviewing);
                        currStat.setReviewFailedCount(reviewFailed);
                    }
                    break;
                    case APPROVAL_AMOUNT: {
                        List<String> projectCodes = projects.stream()
                                .filter(w -> w.getApprovalAmount() != null
                                        && NumberUtil.isGreater(w.getApprovalAmount(), BigDecimal.ZERO))
                                .map(Project::getProjectCode)
                                .collect(Collectors.toList());
                        currStat.setProjectCount(projectCodes.size());
                        Pair<Integer, Integer> systemReplaceCount = countReplaceSystemByProjectCodes(projectCodes);
                        currStat.setSourceCount(systemReplaceCount.getKey());
                        currStat.setTargetCount(systemReplaceCount.getValue());
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap, projectCodes));
                    }
                    break;
                    case PROJECT_APPROVAL: {
                        List<String> projectCodes = getProjectCodes(allProjectCodes, ProjectStateChangeEvent.DECLARED_RECORD_SUBMIT);
                        currStat.setProjectCount(projectCodes.size());
                        Pair<Integer, Integer> systemReplaceCount = countReplaceSystemByProjectCodes(projectCodes);
                        currStat.setSourceCount(systemReplaceCount.getKey());
                        currStat.setTargetCount(systemReplaceCount.getValue());
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap, projectCodes));
                    }
                    break;
                    case PROJECT_PURCHASE: {
                        Wrapper<Purchase> purchaseQuery = Wrappers.lambdaQuery(Purchase.class)
                                .select(Purchase::getId, Purchase::getProjectId, Purchase::getBidType, Purchase::getSupplier)
                                .in(Purchase::getProjectId, allProjectIds);
                        List<Purchase> purchases = purchaseService.list(purchaseQuery);
                        projectPurchaseCountMap.putAll(CollUtils.group(purchases, Purchase::getProjectId));
                        List<Long> projectIds = projectPurchaseCountMap.entrySet().stream()
                                .filter(w -> CollUtil.allMatch(w.getValue(),
                                        p -> StrUtil.isNotBlank(p.getSupplier())))
                                .map(Map.Entry::getKey)
                                .collect(Collectors.toList());
                        currStat.setProjectCount(projectIds.size());
                        Pair<Integer, Integer> systemReplaceCount = countReplaceSystemByProjectIds(projectIds);
                        currStat.setSourceCount(systemReplaceCount.getKey());
                        currStat.setTargetCount(systemReplaceCount.getValue());
                        List<String> projectCodes = CollUtils.convert(projectIds, ProjectIdCodeCacheUtil::get);
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap, projectCodes));
                        if (!purchases.isEmpty()) {
                            Wrapper<PurchaseStatusChange> pQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                                    .select(PurchaseStatusChange::getEvent, PurchaseStatusChange::getProjectId)
                                    .in(PurchaseStatusChange::getProjectId, projectIds)
                                    .in(PurchaseStatusChange::getEvent,
                                            AdaptStateChangeEvent.ADAPT_INFO_PASSED,
                                            SelfTestStateChangeEvent.SELF_TEST_PASSED,
                                            TestValidStateChangeEvent.TEST_VALID_INFO_PASSED,
                                            TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES,
                                            XcfhxStateChangeEvent.XCFHX_APPLY_PASSED,
                                            TenderStateChangeEvent.FINALLY_INSPECTED_PASSED,
                                            TenderStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM,
                                            TenderStateChangeEvent.SUBMIT_PURCHASE_CONSTRUCTION_INFO);
                            List<PurchaseStatusChange> purchaseChanges = purchaseStatusChangeService.list(pQuery);
                            Map<String, Map<Long, Long>> purchaseEventMapTmp = purchaseChanges.stream()
                                    .collect(Collectors.groupingBy(PurchaseStatusChange::getEvent,
                                            Collectors.collectingAndThen(Collectors.toList(),
                                                    w -> CollUtils.groupCount(w, PurchaseStatusChange::getProjectId))));
                            purchaseEventMap.putAll(purchaseEventMapTmp);
                        }
                    }
                    break;
                    case PROJECT_ADAPTION: {
                        List<Long> projectIds = computeProjectIds.apply(TenderStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM, node);
                        currStat.setProjectCount(projectIds.size());
                        Pair<Integer, Integer> replaceSystemCount = countReplaceSystemByProjectIds(projectIds);
                        currStat.setSourceCount(replaceSystemCount.getKey());
                        currStat.setTargetCount(replaceSystemCount.getValue());
                        List<String> projectCodes = CollUtils.convert(projectIds, ProjectIdCodeCacheUtil::get);
                        Map<Long, Long> submitPurcahseRecordMap = purchaseEventMap.get(TenderStateChangeEvent.SUBMIT_PURCHASE_CONSTRUCTION_INFO.name());
                        if (submitPurcahseRecordMap != null) {
                            List<Long> tmpProjectIds = submitPurcahseRecordMap.entrySet().stream()
                                    .filter(w -> w.getValue() > 0)
                                    .map(Map.Entry::getKey)
                                    .collect(Collectors.toList());
                            if (!tmpProjectIds.isEmpty()) {
                                currStat.setAdaptFinishStat(tenderAdaptStatistics(tmpProjectIds));
                            }
                        }
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap, projectCodes));
                    }
                    break;
                    case FIRST_INSPECTED: {
                        List<Long> projectIds = computeProjectIds.apply(TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES, node);
                        Pair<Integer, Integer> replaceSystemCount = countReplaceSystemByProjectIds(projectIds);
                        currStat.setProjectCount(projectIds.size());
                        currStat.setSourceCount(replaceSystemCount.getKey());
                        currStat.setTargetCount(replaceSystemCount.getValue());
                        List<String> projectCodes = CollUtils.convert(projectIds, ProjectIdCodeCacheUtil::get);
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap, projectCodes));
                    }
                    break;
                    case FINAL_INSPECTED: {
                        List<Long> projectIds = computeProjectIds.apply(TenderStateChangeEvent.FINALLY_INSPECTED_PASSED, node);
                        Pair<Integer, Integer> replaceSystemCount = countReplaceSystemByProjectIds(projectIds);
                        currStat.setProjectCount(projectIds.size());
                        currStat.setSourceCount(replaceSystemCount.getKey());
                        currStat.setTargetCount(replaceSystemCount.getValue());
                        List<String> projectCodes = CollUtils.convert(projectIds, ProjectIdCodeCacheUtil::get);
                        currStat.setStoppedCount(stoppedProjectCount(projectStatusChangeMap, projectCodes));
                    }
                    break;
                    default:
                        break;
                }
                retData.add(currStat);
            }
        }
        return retData;
    }

    private TenderAdaptFinishStatVO tenderAdaptStatistics(List<Long> projectIds) {
        Wrapper<PurchaseStatusChange> purchaseQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                .select(PurchaseStatusChange::getEvent, PurchaseStatusChange::getBidId)
                .in(PurchaseStatusChange::getProjectId, projectIds)
                .in(PurchaseStatusChange::getEvent, AdaptStateChangeEvent.SUBMIT_ADAPT_INFO,
                        SelfTestStateChangeEvent.SUBMIT_SELF_TEST_INFO,
                        TestValidStateChangeEvent.SUBMIT_TEST_VALID_INFO,
                        TenderStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
        List<PurchaseStatusChange> statusChanges = purchaseStatusChangeService.list(purchaseQuery);
        Map<String, Integer> eventMap = statusChanges.stream()
                .collect(Collectors.groupingBy(PurchaseStatusChange::getEvent,
                        Collectors.collectingAndThen(Collectors.mapping(PurchaseStatusChange::getBidId, Collectors.toSet()), Set::size)));
        TenderAdaptFinishStatVO stat = new TenderAdaptFinishStatVO();
        stat.setFinishTestValidCount(eventMap.getOrDefault(TestValidStateChangeEvent.SUBMIT_TEST_VALID_INFO.name(), 0));
        stat.setFinishAdaptionInfo(eventMap.getOrDefault(AdaptStateChangeEvent.SUBMIT_ADAPT_INFO.name(), 0));
        stat.setFinishOrgConfirmCount(eventMap.getOrDefault(TenderStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM.name(), 0));
        stat.setFinishSelfTestCount(eventMap.getOrDefault(SelfTestStateChangeEvent.SUBMIT_SELF_TEST_INFO.name(), 0));
        return stat;
    }

    private static int stoppedProjectCount(Map<String, List<ProjectStatusChange>> projectStatusChangeMap, List<String> projectCodes) {
        return CollUtil.count(projectStatusChangeMap.entrySet(),
                w -> projectCodes.contains(w.getKey())
                        && CollUtil.anyMatch(w.getValue(), x -> ProjectStateChangeEvent.STOPPED_APPLY_PASS.eq(x.getEvent())));
    }

    private static int lastMatchProjectCount(Map<String, List<ProjectStatusChange>> projectStatusChangeMap,
                                             ProjectStateChangeEvent... events) {
        List<ProjectStateChangeEvent> changeEvents = Arrays.asList(events);
        return CollUtil.count(projectStatusChangeMap.entrySet(),
                w -> {
                    w.getValue().sort(Comparator.comparing(ProjectStatusChange::getId).reversed());
                    return changeEvents.stream().anyMatch(x -> x.eq(w.getValue().get(0).getEvent()));
                });
    }

    public PageVo<ProjectLibListItemVO> pageProjectLib(WorkbenchProjectLibReq req) {
        LambdaQueryWrapper<Project> query = workbenchProjectLibQuery(req);
        ProjectManageUtil.projectBaseQuery(query);
        projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT);
        switch (req.getProcessNode()) {
            case PROJECT_DECLARED:
                break;
            case PROJECT_REVIEW:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.PROJECT_REVIEW_PASS);
                break;
            case APPROVAL_AMOUNT:
                query.isNotNull(Project::getApprovalAmount).gt(Project::getApprovalAmount, BigDecimal.ZERO);
                break;
            case PROJECT_APPROVAL:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.DECLARED_RECORD_SUBMIT);
                break;
            case PROJECT_PURCHASE:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                break;
            case FIRST_INSPECTED:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES);
                break;
            case PROJECT_ADAPTION:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                break;
            case FINAL_INSPECTED:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.FINAL_ACCEPTANCE_PASS);
                break;
            default:
                return PageVo.empty();
        }
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(),
                w -> ProjectLibListItemVO.builder()
                        .id(w.getId())
                        .fromType("1")
                        .projectName(w.getProjectName())
                        .projectCode(w.getProjectCode())
                        .projectType(w.getProjectType())
                        .projectYear(w.getProjectYear())
                        .buildOrg(w.getBuildOrgName())
                        .buildOrgCode(w.getBuildOrgCode())
                        .approvedAmount(w.getApprovalAmount())
                        .reviewAmount(w.getReviewAmount())
                        .declaredAmount(w.getDeclareAmount())
                        .createOn(w.getCreateOn())
                        .stage(w.getStage())
                        .status(w.getStatus())
                        .build());
        return PageVo.of(records, page.getTotal());
    }

    public SystemReplaceInfoVO listSystemReplaceInfos(WorkbenchProjectLibReq req) {
        LambdaQueryWrapper<Project> query = workbenchProjectLibQuery(req)
                .select(Project::getProjectCode, Project::getProjectName,
                        Project::getBuildOrgCode, Project::getBuildOrgName);
        projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT);
        switch (req.getProcessNode()) {
            case PROJECT_DECLARED:
                break;
            case PROJECT_REVIEW:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.PROJECT_REVIEW_PASS);
                break;
            case APPROVAL_AMOUNT:
                query.isNotNull(Project::getApprovalAmount).gt(Project::getApprovalAmount, BigDecimal.ZERO);
                break;
            case PROJECT_APPROVAL:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.DECLARED_RECORD_SUBMIT);
                break;
            case PROJECT_PURCHASE:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD);
                break;
            case FIRST_INSPECTED:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES);
                break;
            case PROJECT_ADAPTION:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM);
                break;
            case FINAL_INSPECTED:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.FINAL_ACCEPTANCE_PASS);
                break;
            default:
                return null;
        }
        List<Project> projects = projectService.list(query);
        return buildSystemReplaceInfos(projects);
    }

    private LambdaQueryWrapper<Project> workbenchProjectLibQuery(WorkbenchProjectLibReq req) {
        Assert.notNull(req.getProcessNode(), "节点不能为空");
        Long unitId = Optional.ofNullable(req.getUnitId()).orElseGet(LoginUserUtil::getMhUnitId);
        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(unitId);
        return Wrappers.lambdaQuery(Project.class)
                .eq(req.getProjectYear() != null, Project::getProjectYear, req.getProjectYear())
                .like(StrUtil.isNotBlank(req.getBuildOrg()), Project::getBuildOrgName, req.getBuildOrg())
                .like(StrUtil.isNotBlank(req.getProjectName()), Project::getProjectName, req.getProjectName())
                .in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf))
                .ge(req.getReviewAmountMin() != null, Project::getReviewAmount, req.getReviewAmountMin())
                .le(req.getReviewAmountMax() != null, Project::getReviewAmount, req.getReviewAmountMax())
                .ge(req.getApproveAmountMin() != null, Project::getApprovalAmount, req.getApproveAmountMin())
                .le(req.getApproveAmountMax() != null, Project::getApprovalAmount, req.getApproveAmountMax())
                .eq(req.getUnitStrip() != null, Project::getUnitStrip, req.getUnitStrip())
                .eq(Project::getNewest, Boolean.TRUE)
                .orderByDesc(Project::getUpdateOn);
    }

    private SystemReplaceInfoVO buildSystemReplaceInfos(List<Project> projects) {
        if (projects.isEmpty()) {
            return null;
        }
        List<String> projectCodes = CollUtils.fieldList(projects, Project::getProjectCode);
        Wrapper<ProjectGovSystemReplaceInfos> query = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .in(ProjectGovSystemReplaceInfos::getProjectCode, projectCodes);
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.list(query);
        if (systemReplaceInfos.isEmpty()) {
            return null;
        }
        SystemReplaceInfoVO retDetail = new SystemReplaceInfoVO();
        Map<String, List<ProjectGovSystemReplaceInfos>> projectSystemMap = CollUtils.group(systemReplaceInfos,
                ProjectGovSystemReplaceInfos::getProjectCode);
        Map<String, List<Project>> unitProjectMap = CollUtils.group(projects, Project::getBuildOrgCode);
        List<UnitProjectGovSystemReplaceVO> retList = new ArrayList<>();
        AtomicInteger sourceCount = new AtomicInteger(0);
        AtomicInteger targetCount = new AtomicInteger(0);
        for (Map.Entry<String, List<Project>> entry : unitProjectMap.entrySet()) {
            List<Project> tmpProjects = entry.getValue();
            List<ProjectGovSystemReplaceVO> currOrgReplaceInfos = new ArrayList<>();
            for (Project proj : tmpProjects) {
                List<ProjectGovSystemReplaceInfos> replaceInfos = projectSystemMap.get(proj.getProjectCode());
                if (replaceInfos != null) {
                    ProjectGovSystemReplaceVO projectSystem = new ProjectGovSystemReplaceVO();
                    projectSystem.setProjectName(proj.getProjectName());
                    currOrgReplaceInfos.add(projectSystem);
                    Map<Integer, List<ProjectGovSystemReplaceInfos>> replaceInfoGroup = CollUtils.group(replaceInfos,
                            ProjectGovSystemReplaceInfos::getBatchNo);
                    List<GovSystemReplaceInfoDTO> replaceInfoList = replaceInfoGroup.entrySet().stream()
                            .sorted(Comparator.comparingInt(Map.Entry::getKey))
                            .map(w -> {
                                List<ProjectGovSystemReplaceInfos> currBatchValues = w.getValue();
                                ProjectGovSystemReplaceInfos firstReplaceInfo = currBatchValues.get(0);
                                GovSystemReplaceInfoDTO replaceInfo = new GovSystemReplaceInfoDTO();
                                replaceInfo.setReplaceType(firstReplaceInfo.getReplaceType());
                                replaceInfo.setTargetSystemName(firstReplaceInfo.getTargetSystem());
                                List<String> sourceSystemNames = CollUtils.fieldList(currBatchValues, val -> {
                                    String sourceSystem = val.getSourceSystem();
                                    if (isValidSystem.test(sourceSystem)) {
                                        sourceCount.incrementAndGet();
                                    }
                                    return sourceSystem;
                                });
                                replaceInfo.setSourceSystemNames(sourceSystemNames);
                                if (isValidSystem.test(replaceInfo.getTargetSystemName())) {
                                    targetCount.incrementAndGet();
                                }
                                return replaceInfo;
                            }).collect(Collectors.toList());
                    projectSystem.setReplaceSystems(replaceInfoList);
                }
            }
            if (!currOrgReplaceInfos.isEmpty()) {
                UnitProjectGovSystemReplaceVO item = new UnitProjectGovSystemReplaceVO();
                item.setUnitName(tmpProjects.get(0).getBuildOrgName());
                item.setProjectSystemReplaces(currOrgReplaceInfos);
                retList.add(item);
            }
        }
        retDetail.setBeforeReplace(sourceCount.get());
        retDetail.setAfterReplace(targetCount.get());
        retDetail.setReplaceDetail(retList);
        return retDetail;
    }
}
